class DatabaseConnector:
    def __init__(self, connection, cursor):
        self.connection = connection
        self.cursor = cursor

    def __del__(self):
        """
        When this object is being deleted, close the connection
        """
        if self.connection:
            self.close()

    def __enter__(self):
        """
        Allows a DatabaseConnector to be used as a context manager
        """
        return self

    def __exit__(self, type, value, traceback):
        """
        On exit as a context manager, close the connection
        """
        self.close()

    @classmethod
    def connect_mysql(cls, host='', username='', password='', port=3306, database=''):
        """
        Initializes a MySQL connection with the provided credentials.

        @param host -- the host to connect to (default localhost)
        @param username -- the username to login as (optional)
        @param password -- the password to login with (optional)
        @param port -- the port to connect to (default 3306)
        @param database -- the name of the database to connect to (optional)
        """
        try:
            import MySQLdb as mysql
        except ImportError:
            raise ImportError("Please install the mysql-python package")

        connection = mysql.connect(
            host=host,
            user=username,
            passwd=password,
            port=port,
            db=database
        )
        c = connection.cursor()
        return cls(connection, c)

    @classmethod
    def connect_sqlite(cls, database=':memory:'):
        """
        Initializes a SQLite connection to the given database.

        @param database -- the name of the database file to connect to (default database in RAM)
        """
        import sqlite3

        connection = sqlite3.connect(database=database)
        c = connection.cursor()
        return cls(connection, c)

    @classmethod
    def connect_postgres(cls, host='', username='', password='', port=5432, database=''):
        """
        Initializes a PostgreSQL connection with the provided credentials.

        @param host -- the host to connect to (default Unix Socket)
        @param username -- the username to login as (optional)
        @param password -- the password to login with (optional)
        @param port -- the port to connect to (default 5432)
        @param database -- the name of the database to connect to (optional)
        """
        try:
            import psycopg2
        except ImportError:
            raise ImportError("Please install the mysql-python package")

        connection = psycopg2.connect(
            host=host,
            user=username,
            password=password,
            port=port,
            database=database
        )
        c = connection.cursor()
        return cls(connection, c)

    def query(self, query):
        """
        Returns the result of running the query on the database

        @param query -- the query to pass to the connection

        @return (Tuple<String>) the result of the query as a tuple of Strings
        """
        self.cursor.execute(query)
        return self.cursor.fetchall()

    def execute(self, query):
        """
        Runs a query on the database, but doesn't return the result of the query. Does the
        same thing as query otherwise. Useful for SQL executions such as INSERT or DROP.

        @param query -- the query to execute on the connection.
        """
        self.query(query)

    def compute(self, query):
        """
        Returns the first row of running the query on the database. Useful for SQL aggregate
        queries such as COUNT, MAX, MIN, AVG, FIRST, LAST, or SUM.

        @param query -- the query to pass to the connection

        @return (Tuple<String>) the the first row of the query as a tuple of Strings. Although
        most results will be numerical, 
        """
        rows = self.query(query)
        return rows[0]

    def close(self):
        """
        Closes the connection, preventing any further queries
        """
        self.connection.close()


import socket, json

class SocketConnector:
    def __init__(self, host, port, bufsize=4096):
        """
        Initializes a socket connection with the provided server.

        @param host -- the host to connect to
        @param port -- the port to connect to
        @param bufsize -- the size of data allowed to be received (default 4096)
        """
        self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.address = (host, port)
        self.bufsize = bufsize

    def send(self, data):
        """
        Sends the data to the server as a JSON string and returns the response.

        @param data -- a Python object to send as JSON

        @return (object) the response from the server as a Python object extracted from
            the JSON formatted string
        """
        self.socket.connect(self.address)
        self.socket.send(json.dumps(data))
        response = self.socket.recv(self.bufsize)
        self.socket.close()
        return json.loads(response)


from distutils.core import setup

with open('DESCRIPTION.rst') as f:
    long_des = f.read()

setup(
    name='servconn',
    version='1.0.2',
    author=u'Brandon Chinn',
    author_email='brandonchinn178@gmail.com',
    packages=['servconn'],
    url='http://github.com/brandonchinn178/servconn',
    description='Defines classes that wraps connections to servers',
    long_description=long_des,
    zip_safe=False
)